﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace OSNSystem
{
    public partial class SensorConfig : Form
    {
        public SensorConfig()
        {
            InitializeComponent();
          
        }
        
        /// <summary>
        /// 加载传感器配置界面，将各个节点所接的传感器信息显示在节点相应的位置处
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SensorConfig_Load(object sender, EventArgs e)
        {
            try
            {
                //设置treeview控件默认是全部展开的属性
                treeView1.ExpandAll();
                treeView1.ImageList = imageList1;
               // treeView1.Nodes[0].ImageIndex = 0;
                listDeviceData(searchDeviceData());

                listConfigData(searchConfigData());

               // treeView1.Font = new Font(treeView1.Font.FontFamily, 14.5f);//设置treeview控件字体大小
            }
            catch (Exception exp)
            {
                LogClass.WriteLogFile("传感器配置界面加载异常！" + exp.Message);
            }
        }       
        /// <summary>
        /// 传感器设备添加行号操作
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dataGridView1_RowPostPaint(object sender, DataGridViewRowPostPaintEventArgs e)
        {
            Color color = dataGridView1.RowHeadersDefaultCellStyle.ForeColor;
            if (dataGridView1.Rows[e.RowIndex].Selected)
                color = dataGridView1.RowHeadersDefaultCellStyle.SelectionForeColor;
            else
                color = dataGridView1.RowHeadersDefaultCellStyle.ForeColor;

            using (SolidBrush b = new SolidBrush(color))
            {
                e.Graphics.DrawString((e.RowIndex + 1).ToString(), e.InheritedRowStyle.Font, b, e.RowBounds.Location.X + 20, e.RowBounds.Location.Y + 6);
            }
        }

        /// <summary>
        /// 查找后台设备数据库表格存入datatable表格中
        /// </summary>
        /// <returns></returns>
        private DataTable searchDeviceData()
        {
            string strsql = null;
            try
            {
                DataTable dtDeviceData = null;
                //1.查询所有设备信息
                strsql = "(select deviceId as '设备编码',";
                strsql += "deviceName as '设备名称',";
                strsql += "deviceComm  as '通信方式', deviceIP as '设备IP', deviceElec as '电气参数', deviceProtocol as '协议形式', srcId as '所属数据采集器' from seafloor_device )";

                dtDeviceData = SqlHelper.ExecuteDataTable(strsql);
                return dtDeviceData.Copy();
            }
            catch (Exception exp)
            {
                // MessageBox.Show("查询采集数据出现异常！" + exp.Message);
                LogClass.WriteLogFile("查询设备数据出现异常！" + exp.Message);
                return null;
            }
        }

        /// <summary>
        ///   传感器设备信息页--实时显示设备信息页
        ///   这个相当于可以初始化datagridview初始化的时候可以初始化多少行空格
        /// </summary>
        /// <param name="dt"></param>
        private  void listDeviceData(DataTable dt)
        {
            try
            {
                dataGridView1.Columns.Clear();
                dataGridView1.DataSource = dt;
                dataGridView1.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
                dataGridView1.ReadOnly = true;
                dataGridView1.AllowUserToAddRows = false;
                dataGridView1.AllowUserToDeleteRows = false;
                dataGridView1.Columns[1].Width = 150;

                //Application.DoEvents();

                //while (dataGridView1.Rows.Count < 5)
                //    ((DataTable)dataGridView1.DataSource).Rows.Add();

                for (int i = 0; i < dataGridView1.Columns.Count; i++)
                {
                   dataGridView1.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
                }

            }
            catch (Exception exp)
            {
                LogClass.WriteLogFile("海底观测网拓扑图设备信息表显示异常！" + exp.Message);
            }
        }





        /*****************************************/
        /// <summary>
        /// 查找后台设备数据库表格存入datatable表格中
        /// </summary>
        /// <returns></returns>
        private DataTable searchConfigData()
        {
            string strsql = null;
            try
            {
                DataTable dtDeviceData = null;
                //1.查询所有设备信息
                strsql = "(select deviceId as '设备编码',";
                strsql += "srcId as '数据采集器编码',";
                strsql += "srcCom  as '数据采集器接口', jBId as '接驳盒编码', jBCom as '接驳盒接口' from seafloor_deviceConfig )";

                dtDeviceData = SqlHelper.ExecuteDataTable(strsql);
                return dtDeviceData.Copy();
            }
            catch (Exception exp)
            {
                // MessageBox.Show("查询采集数据出现异常！" + exp.Message);
                LogClass.WriteLogFile("查询设备配置信息表出现异常！" + exp.Message);
                return null;
            }
        }

        /// <summary>
        ///   传感器设备信息页--实时显示设备信息页
        ///   这个相当于可以初始化datagridview初始化的时候可以初始化多少行空格
        /// </summary>
        /// <param name="dt"></param>
        private void listConfigData(DataTable dt)
        {
            try
            {
                dataGridView2.Columns.Clear();
                dataGridView2.DataSource = dt;
                dataGridView2.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
                dataGridView2.ReadOnly = true;
                dataGridView2.AllowUserToAddRows = false;
                dataGridView2.AllowUserToDeleteRows = false;
                dataGridView2.Columns[1].Width = 150;

                //Application.DoEvents();

                while (dataGridView2.Rows.Count < 5)
                    ((DataTable)dataGridView2.DataSource).Rows.Add();

                for (int i = 0; i < dataGridView2.Columns.Count; i++)
                {
                    dataGridView2.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
                }

            }
            catch (Exception exp)
            {
                LogClass.WriteLogFile("设备配置信息表显示异常！" + exp.Message);
            }
        }
        /// <summary>
        /// 设备配置动态表格添加行号
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dataGridView2_RowPostPaint(object sender, DataGridViewRowPostPaintEventArgs e)
        {
            Color color = dataGridView2.RowHeadersDefaultCellStyle.ForeColor;
            if (dataGridView2.Rows[e.RowIndex].Selected)
                color = dataGridView2.RowHeadersDefaultCellStyle.SelectionForeColor;
            else
                color = dataGridView2.RowHeadersDefaultCellStyle.ForeColor;

            using (SolidBrush b = new SolidBrush(color))
            {
                e.Graphics.DrawString((e.RowIndex + 1).ToString(), e.InheritedRowStyle.Font, b, e.RowBounds.Location.X + 20, e.RowBounds.Location.Y + 6);
            }
        }



    }
}
