﻿/*
 * 时间：2020年12月21日16:02:27
 * 功能：处理北斗接收漂流浮标数据
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace OSNSystem
{
    class ThreadProcessBeidou
    {
        Main f;//用于委托回调显示
        string content;//读到的原始串口数据

        public ThreadProcessBeidou(Main frm, string mesg)
        {
            f = frm;
            content = mesg;
            showBaiducallback = new delShowBaidu(showBaiduMethod);
        }




        /**以下为在百度地图上进行显示的委托回调函数及相关定义***/
        private delegate void delShowBaidu(string a, string b);
        private delShowBaidu showBaiducallback;

        /// <summary>
        /// 按照上传上来的经纬度信息，将经纬度信息定位点显示在百度地图对应位置
        /// </summary>
        /// <param name="a"></param>经度信息
        /// <param name="b"></param>纬度信息
        private void showBaiduMethod(string a, string b)
        {
            try
            {
                f.webGPS.Document.InvokeScript("clear");
                f.webGPS.Document.InvokeScript("setLocation", new object[] { a, b });//在对应位置显示坐标
                f.txtLongitude.Text = Convert.ToDouble(a).ToString();
                f.txtLatitude.Text = Convert.ToDouble(b).ToString();

            }
            catch (Exception exp)
            {
                MessageBox.Show("在百度地图上进行图标显示异常！" + exp.Message);
            }
        }



        /// <summary>
        /// 新增处理GPS位置信息函数
        /// </summary>
        public  void dealGPS( )
        {
            try
            {
                ////string message = tempMessage.ToString();
                //byte[] data = Encoding.Default.GetBytes(message);//转换成对应的字节


                //string content = null;
                //for (int k = 0; k < data.Length; k++)
                //    content += String.Format("{0:X2}", data[k]);//转换成对应的16进制的字符串

                //1.利用16进制字符串判断长度是否符合要求

                byte[] data = StrToHexByte(content);//将十六进制字符串转换成对应的数存储在字节数组中

                int length = Convert.ToInt32(content.Substring(10, 4), 16);//从数据包中截取出自带的长度信息；

                if (length != content.Length / 2)//判断数据包中带的长度信息与实际的长度信息是否一致
                {
                    //若是长度信息不一致，则认为错误
                }
                else//长度信息一致，判断校验值是否正确
                {
                    byte[] tempData = new byte[data.Length - 1];
                    for (int i = 0; i < data.Length - 1; i++)
                    {
                        tempData[i] = data[i];
                    }
                    string xorHex = HexXorByte(tempData);
                    if (Convert.ToInt32( xorHex,16) == Convert.ToInt32( content.Substring(content.Length - 2, 2),16))//如果异或校验值通过
                    {
                        int lon1 = Convert.ToInt32(content.Substring(40, 2), 16);//经度部分的整数部分
                        int lon2 = Convert.ToInt32(content.Substring(42, 8), 16);//经度部分的小数部分

                        byte[] L = new byte[1];
                        L[0] = data[25];
                        string longitude = lon1.ToString() + "." + lon2.ToString().ToString();//经度值显示

                        int lat1 = Convert.ToInt32(content.Substring(52, 2), 16);//纬度部分的整数部分
                        int lat2 = Convert.ToInt32(content.Substring(54, 8), 16);//纬度部分的小数部分


                        string latitude = lat1.ToString() + "." + lat2.ToString();//纬度值显示




                        f.Invoke(showBaiducallback, longitude, latitude);


                    }
                    else //异或校验值不通过
                    {
                        //直接舍弃不做任何处理
                    }
                }

                //以下是将经纬度点加载到百度地图中
                //f.webGPS.Document.InvokeScript("setLocation", new object[] { lon, lat });//创建标记
                //f.webGPS.Document.InvokeScript("clear");//清除标记

                //f.Invoke(showBaiducallback,lon,lat);//c#与js交互代码，其中js交互部分一定要放到委托中使用，否则程序异常！


            }
            catch (Exception exp)
            {
                LogClass.WriteLogFile("处理GPS位置信息出现异常！" + exp.Message);
            }
        }



        /// <summary>
        /// 求解字节数组异或完成之后的十六进制字符串形式
        /// </summary>
        /// <param name="hex"></param>
        /// <returns></returns>
        private  string HexXorByte(byte[] hex)
        {
            int a = 0;
            string str = "";
            for (int i = 0; i < hex.Length - 1; i++)
            {
                a ^= hex[i];
            }
            str = Convert.ToString(a, 16);
            return str;
        }



        /// <summary>
        /// 将十六进制字符串转换成字节类型存储在数组中
        /// </summary>
        /// <param name="val"></param>
        /// <returns></returns>
        private byte[] StrToHexByte(string val)
        {
            byte[] bytes = new byte[val.Length / 2];

            for (int i = 0; i < bytes.Length; i++)
            {
                bytes[i] = Convert.ToByte(val.Substring(i * 2, 2), 16);
            }
            return bytes;
        }
    }
}
